from __future__ import annotations

import sys
import json
import logging
import argparse
from dataclasses import asdict

from .logger import LOGGER, CONSOLE_LOGGER
from .template import Template
from .generate_csv import generate_csv
from .generate_shots import generate_shots


def main():
    parser = argparse.ArgumentParser(
        description='Generate dummy EXR files following VFX naming conventions.',
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        usage="""
Examples:
  fakeshot --out ./renders
    Generate using default template to ./renders directory

  fakeshot --show myfilm --scene 010,020 --shot 0010,0020 --task comp,anim
    Generate shots for 'myfilm' with multiple scenes, shots, and tasks

  fakeshot --end_frame 1024 --width 1920 --height 1080
    Generate HD frames from 1001 to 1023

  fakeshot --template custom.json --out /path/to/output
    Use custom template file

  fakeshot --sample-template > my_template.json
    Generate a sample template file
        """.strip()
    )

    parser.add_argument('-v', '--verbose', action='store_true', help='Verbose mode')
    parser.add_argument('--out', help='Output directory for EXR files. (default: ./out)')
    parser.add_argument('--template', help='Specify a different template.json')
    parser.add_argument('--sample-template', action='store_true', help='Output a sample template')
    parser.add_argument('--csv', action='store_true',
                        help='Export as csv rather than creating the shots.')

    template = Template()

    parser.add_argument(
        '--show', default=template.show,
        help="Project/show name (e.g., 'prj', 'film_abc')"
    )
    parser.add_argument(
        '--scene', default=template.scene,
        help="Scene identifier(s), comma-separated (e.g., '001', '001,002,003')"
    )
    parser.add_argument(
        '--shot', default=template.shot,
        help="Shot identifier(s), comma-separated (e.g., '0010', '0010,0020,0030')"
    )
    parser.add_argument(
        '--task', default=template.task,
        help="Task type(s), comma-separated (e.g., 'comp', 'comp,anim,lighting')"
    )
    parser.add_argument(
        '--version', default=template.version,
        help="Version identifier(s), comma-separated (e.g., 'v001', 'v001,v002')"
    )
    parser.add_argument(
        '--start_frame', type=int, default=template.start_frame,
        help='Starting frame number (inclusive)'
    )
    parser.add_argument(
        '--end_frame', type=int, default=template.end_frame,
        help='Ending frame number (exclusive)'
    )
    parser.add_argument(
        '--width', type=int, default=template.width,
        help='Image width in pixels'
    )
    parser.add_argument(
        '--height', type=int, default=template.height,
        help='Image height in pixels'
    )
    parser.add_argument(
        '--path-pattern', default=template.path_pattern,
        help='Directory structure pattern tokens'
    )
    parser.add_argument(
        '--file-pattern', default=template.file_pattern,
        help='Filename pattern tokens'
    )

    args = parser.parse_args()

    if args.verbose:
        CONSOLE_LOGGER.setLevel(logging.DEBUG)

    if args.sample_template:
        print(json.dumps(asdict(template), indent=4))
        sys.exit(0)

    # Apply CLI arguments to template
    for attr, value in args._get_kwargs():
        if hasattr(template, attr):
            setattr(template, attr, value)

    if args.template:
        LOGGER.info('Using user template: %s', args.template)
        with open(args.template) as f:
            template.update(json.load(f))

    if args.csv:
        generate_csv(template, args.out)
        sys.exit(0)

    generate_shots(template, args.out or './out')

    sys.exit(0)


if __name__ == '__main__':
    main()



from __future__ import annotations

import numpy as np
import OpenEXR
from PIL import Image, ImageDraw, ImageFont


def generate_exr(
    *,
    path: str,
    label: str,
    width: int = 512,
    height: int = 512
) -> None:
    """
    Generates a dummy EXR image file with a centered label.

    ```
    generate_dummy_exr(
        path='/path/to/file.1001.exr',
        label='myfile.1001',
        width=1920,
        height=1080
    )
    ```

    Args:
        path (str): The file path where the EXR image will be saved.
        label (str): The text label to display in the center of the image.
        width (int, optional): The width of the image in pixels. Defaults to 512.
        height (int, optional): The height of the image in pixels. Defaults to 512.

    Returns:
        None
    """
    #  Create a grayscale pillow image
    img = Image.new('RGB', (width, height), color=(20, 20, 20))  # dark gray background
    draw = ImageDraw.Draw(img)

    font = ImageFont.load_default(24)

    bbox = draw.textbbox((0, 0), label, font=font)
    text_w = bbox[2] - bbox[0]
    text_h = bbox[3] - bbox[1]
    text_position = ((width - text_w) // 2, (height - text_h) // 2)
    draw.text(text_position, label, font=font, fill=(220, 220, 220))

    # Convert Pillow image to NumPy
    img_np = np.asarray(img).astype(np.float32) / 255.0  # Normalize to 0.0–1.0

    # Create EXR header + save
    header = OpenEXR.Header(width, height)
    exr = OpenEXR.OutputFile(path, header)

    # Split RGB channels
    exr.writePixels({
        'R': img_np[:, :, 0].tobytes(),
        'G': img_np[:, :, 1].tobytes(),
        'B': img_np[:, :, 2].tobytes()
    })

    exr.close()



from __future__ import annotations

from typing import Any, Iterator, Generator
from pathlib import Path
from itertools import product
from dataclasses import asdict, dataclass


def split_n_strip(s: str, delimiter: str = ',') -> list[str]:
    """Split by delimiter and strip white space."""
    return [s.strip() for s in s.split(delimiter)]


@dataclass(frozen=True)
class Shot:
    show: str
    scene: str
    shot: str
    task: str
    version: str
    frame: int

    def generate_path(self, template: Template, output: str) -> Path:
        data = asdict(self)
        data['frame'] = f'{self.frame:04d}'

        path = f'{template.path_pattern}/{template.file_pattern}.exr'
        return Path(output, path.format(**data))


@dataclass(kw_only=True)
class Template:
    show: str = 'prj'
    scene: str = '001,002'
    shot: str = '0010,0020'
    task: str = 'comp,anim'
    version: str = 'v001'
    start_frame: int = 1001
    end_frame: int = 1010
    width: int = 512
    height: int = 512
    path_pattern: str = '{show}/{scene}/{shot}/{task}/{version}'
    file_pattern: str = '{show}_{scene}_{shot}_{task}_{version}.{frame}'

    def _get_permutations(self) -> Iterator[Any]:
        """
        Generates all possible permutations of scene, shot, task, version, and frame range.

        Yields:
            Iterator[Any]: An iterator over tuples containing all combinations
            of the split values of show, scene, shot, task, version, and
            each frame in the specified range.
        """
        return product(
            [self.show],
            self.get_scenes(),
            self.get_shots(),
            self.get_tasks(),
            self.get_versions(),
            range(self.start_frame, self.end_frame),
        )

    def get_scenes(self) -> list[str]:
        return split_n_strip(self.scene)

    def get_shots(self) -> list[str]:
        return split_n_strip(self.shot)

    def get_tasks(self) -> list[str]:
        return split_n_strip(self.task)

    def get_versions(self) -> list[str]:
        return split_n_strip(self.version)

    def generate_shots_permutations(self) -> Generator[Shot, Any, None]:
        """Returns a generator of Shot objects based on the current template data."""
        for _shot_permutation in self._get_permutations():
            yield Shot(*_shot_permutation)

    def update(self, user_template: dict[str, Any]):
        """Update current instance with dictionary data."""
        for k, v in user_template.items():
            setattr(self, k, v)



from __future__ import annotations

from .logger import LOGGER
from .template import Template
from .generate_exr import generate_exr


def generate_shots(template: Template, output: str) -> None:
    """
    Generates dummy EXR files for all permutations defined by the template.

    This function creates directories and generates dummy EXR files for each combination
    of scene, shot, task, version, and frame as specified by the template.

    ```
    template = Template(...)
    generate_dummy_shots(template, './data')
    ```

    Args:
        template (Template): The template object containing shot parameters and file patterns.
        output (str): The root directory where the generated EXR files will be saved.

    Returns:
        None
    """
    LOGGER.info('Generating shots...')
    for shot in template.generate_shots_permutations():

        shot_path = shot.generate_path(template, output)
        shot_path.parent.mkdir(exist_ok=True, parents=True)

        LOGGER.debug('Generating: %s', shot_path)

        generate_exr(
            path=shot_path.as_posix(),
            label=shot_path.stem,
            width=template.width,
            height=template.height
        )

    LOGGER.info('EXR files generated.')



from __future__ import annotations

import io
import csv
from itertools import product

from .template import Template


def generate_csv(template: Template, output: str = '') -> None:
    """
    Generates a CSV file containing all combinations of scenes, shots, and tasks
    from the provided template.

    Args:
        template (Template): A Template object
        output (str, optional): The directory path where the 'shots.csv' file
        will be saved. If not provided, the CSV content will be printed to stdout.

    Returns:
        None

    """
    out = open(f'{output}/shots.csv', 'w', newline='') if output else io.StringIO()

    writer = csv.DictWriter(out, fieldnames=['scene', 'shot', 'task'])
    writer.writeheader()

    scenes = template.get_scenes()
    shots = template.get_shots()
    tasks = template.get_tasks()

    for scene, shot, task in product(scenes, shots, tasks):
        writer.writerow(dict(scene=scene, shot=shot, task=task))

    if isinstance(out, io.StringIO):
        print(out.getvalue())

    out.close()




from __future__ import annotations

import sys
import logging

LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.DEBUG)

CONSOLE_LOGGER = logging.StreamHandler(stream=sys.stdout)
CONSOLE_LOGGER.setLevel(logging.INFO)
CONSOLE_LOGGER.setFormatter(
    logging.Formatter('%(levelname)-8s - %(message)s')
)

LOGGER.addHandler(CONSOLE_LOGGER)



"""
A package for generating dummy EXR files following VFX naming conventions.

Basic usage:
    from fakeshot import Template, generate_dummy_shots

    template = Template(show='my_project')
    generate_dummy_shots(template, './output')
"""

from __future__ import annotations

from .template import Template
from .generate_csv import generate_csv
from .generate_exr import generate_exr
from .generate_shots import generate_shots

__all__ = [
    'Template',
    'generate_csv',
    'generate_shots',
    'generate_exr'
]

__version__ = '0.3.2'
